@@ -1,6 +1,6 @@
 // -*- mode: cpp; mode: fold -*-
 // Description								/*{{{*/
-// $Id: columnbar.h,v 1.8 1998/01/24 02:54:55 jgg Exp $
+// $Id: columnbar.h,v 1.2 1998/08/27 04:09:38 jgg Exp $
 /* ######################################################################
 
    Column Bar - A horizontal bar providing titles for columns
@@ -25,6 +25,10 @@
 #ifndef DEITY_COLUMNBAR_H
 #define DEITY_COLUMNBAR_H
 
+#ifdef __GNUG__
+#pragma interface "deity/columnbar.h"
+#endif  
+
 #include <deity/basic.h>
 
 class ColumnBar : public BasicWidget
