@@ -1,6 +1,6 @@
 // -*- mode: cpp; mode: fold -*-
 // Description								/*{{{*/
-// $Id: gpmdev.h,v 1.5 1997/08/18 20:49:35 tom Exp $
+// $Id: gpmdev.h,v 1.2 1998/08/27 04:09:44 jgg Exp $
 /* ######################################################################
 
    GPMDev - GPM based mouse input handler
@@ -16,6 +16,10 @@
 #ifndef DEITY_GPMDEV_H
 #define DEITY_GPMDEV_H
 
+#ifdef __GNUG__
+#pragma interface "deity/gpmdev.h"
+#endif  
+
 #include <deity/selectloop.h>
 
 class Widget;
