@@ -1,6 +1,6 @@
 // -*- mode: cpp; mode: fold -*-
 // Description								/*{{{*/
-// $Id: x11dev.cc,v 1.36 1998/06/21 03:25:13 jgg Exp $
+// $Id: x11dev.cc,v 1.7 1998/09/15 05:57:27 jgg Exp $
 /* ######################################################################
    
    X11Dev - X Windows Device Interface
@@ -8,12 +8,16 @@
    ###################################################################### */
 									/*}}}*/
 // Include Files							/*{{{*/
+#ifdef __GNUG__
+#pragma implementation "deity/x11dev.h"
+#endif  
 #include <iostream.h>
 #include <time.h>
 #include <stdio.h>
 #include <stdlib.h>
 
 #include <deity/widget.h>
+#include <deity/widget-thread.h>
 #include <deity/window.h>
 #include <deity/utils.h>
 
@@ -76,7 +80,7 @@ XWindowsGC::XWindowsGC()
    LastColorInsert = 0;
 }
 									/*}}}*/
-// XWindows::~XWindowsGC - Destructor					/*{{{*/
+// XWindowsGC::~XWindowsGC - Destructor					/*{{{*/
 // ---------------------------------------------------------------------
 /* */
 XWindowsGC::~XWindowsGC()
@@ -513,7 +517,11 @@ Rect XWindowsGC::ExtentText(const char *Text,unsigned int Len)
    int Descent;
    XCharStruct S;
    XTextExtents(XFont,Text,Len,&Dir,&Ascent,&Descent,&S);
-   return Rect(S.lbearing,S.ascent,S.width,Ascent+Descent);
+   if (XFont != 0)
+      return Rect(S.lbearing,XFont->max_bounds.ascent,S.width,
+		  XFont->max_bounds.ascent+XFont->max_bounds.descent);
+   else
+      return Rect(S.lbearing,S.ascent,S.width,Ascent+Descent);
 }
 									/*}}}*/
 // XWindowsGC::TextTranslate - Translate helper for DrawText		/*{{{*/
@@ -536,30 +544,6 @@ Rect XWindowsGC::TextTranslate(Point &P,const char *Text,unsigned int Len,
    return Res;
 }
 									/*}}}*/
-// XWindowsGC::MaskFill - Fills an outer rectangle without filling inner/*{{{*/
-// ---------------------------------------------------------------------
-/* This is used by the text and bitmap rendering routines to provide a 
-   solid background colour beyond the rendering area of the actual
-   item. */
-void XWindowsGC::MaskFill(Rect Outer,Rect Inner)
-{
-   // Fill the remainder of the rectangle not covered by the text..
-   Inner.Clip(Outer);
-   XSetForeground(*this,CurGC,BackColor);
-   if (Inner.x != 0)
-      Fill(Rect(Outer.x,Outer.y,Inner.x,Outer.h));
-   if (Inner.y != 0)
-      Fill(Rect(Outer.x,Outer.y,Outer.w,Inner.y));
-   if (Inner.x + Inner.w != Outer.w)
-      Fill(AbsRect(Inner.x + Inner.w + Outer.x,Outer.y,
-		   Outer.x + Outer.w,Outer.y + Outer.h));
-   if (Inner.y + Inner.h != Outer.h)
-      Fill(AbsRect(Outer.x,Inner.y + Inner.h + Outer.y,
-		   Outer.x + Outer.w,Outer.y + Outer.h));
-
-   XSetForeground(*this,CurGC,ForeColor);
-}
-									/*}}}*/
 // XWindowsGC::DrawTextT - Draw a transparent text string		/*{{{*/
 // ---------------------------------------------------------------------
 /* This draws a text string with a clear background. */
@@ -596,11 +580,11 @@ void XWindowsGC::DrawText(Rect Loc,Point Pos,const char *Text,unsigned int Len,
       CRect.height = Tmp.h;
       XSetClipRectangles(*this,CurGC,Clip.x,Clip.y,&CRect,1,Unsorted);
    }
-   
+
    // Draw the string
    XDrawImageString(*this,CurWin,CurGC,Clip.x + Pos.x + Loc.x,
 		    Clip.y + Pos.y + Loc.y + XFont->ascent,Text,Len);
-
+   
    // Restore the clipping rectangle
    if (TR.Inside(Loc) == false)
    {
@@ -615,7 +599,9 @@ void XWindowsGC::DrawText(Rect Loc,Point Pos,const char *Text,unsigned int Len,
    // Compensate for clipping of one of the coord pairs.
    TR.x += Loc.x - Tmp.x;
    TR.y += Loc.y - Tmp.y;
+   XSetForeground(*this,CurGC,BackColor);
    MaskFill(Tmp,TR);
+   XSetForeground(*this,CurGC,ForeColor);
 }
 									/*}}}*/
 // XWindowsGC::DrawText - Draw a text string				/*{{{*/
@@ -637,7 +623,10 @@ void XWindowsGC::DrawText(Point Pos,const char *Text,unsigned int Len,
 unsigned int XWindowsGC::WrapText(const char *Start,unsigned int Len,long Max)
 {
    if (XFont == 0)
+   {
+      clog << "Text extenting failed" << endl;
       return 0;
+   }
    
    long Count = 0;
    const char *I;
@@ -683,7 +672,7 @@ bool XWindowsFd::Read(SelectLoop &)
    XEvent ev;
 
    double C = GetCurrentTime();
-   GrabMutex WidgetLock(Widget::WidgetLock);
+   Widget::Lock Lock;
 
    while (1)
    {
@@ -713,6 +702,9 @@ bool XWindowsFd::Read(SelectLoop &)
 	    
 	    // Locate the window the notification was for
 	    Widget *Target = LocateWidget((void *)ev.xclient.window);
+	    if (Target == 0)
+	       break;
+	    
 	    if (Target->Trigger(Nt_CloseReq,0) == true)
 	       exit(0);
 	    
@@ -740,8 +732,8 @@ bool XWindowsFd::Read(SelectLoop &)
 	    Widget *Target = LocateWidget((void *)ev.xexpose.window);
 	    if (Target == 0)
 	       break;
-	    Target->Draw();
-//	    Target->Damage();
+//	    Target->Draw();
+	    Target->Damage();
 	    break;
 	 }
 	 
@@ -806,8 +798,12 @@ bool XWindowsFd::Read(SelectLoop &)
 /* */
 void XWindowsFd::Setup(SelectLoop &Loop)
 {
-   Root->RealizeFamily();
-   GenGC::GC->Flush();
+   {      
+      Widget::Lock Lock;
+   
+      Root->RealizeFamily();
+      GenGC::GC->Flush();
+   }
    
    // There may be some events that were internally queued before we got here
    Read(Loop);   
